package com.taauav.admin.controller;


import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.common.constant.SysPositionConstant;
import com.taauav.admin.entity.SysPosition;
import com.taauav.admin.service.ISysPositionService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Map;

/**
 * <p>
 * 职位表 前端控制器
 * </p>
 *
 * @author daixt
 * @since 2019-11-12
 */
@RestController
@RequestMapping("/position")
public class SysPositionController {
    @Autowired
    private ISysPositionService positionService;
    @Resource
    private Response response;

    private static final String controllerName = "position";

    /**
     * 岗位列表接口
     * @param map
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) Map<String, String> map) {
        return positionService.index(map);
    }

    /**
     * 编辑-提交接口
     * @param map
     * @return
     */
    @PostMapping(value = "/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody SysPosition map) {
        return positionService.edit(map);
    }

    /**
     * 编辑-获取数据
     * @param id
     * @return
     */
    @GetMapping("/getPositionInfo")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response getPositionInfo(Integer id) {
        if (id < 1) {
            return response.failure("岗位id必须为正整数");
        }
        return positionService.getPositionInfo(id);
    }

    /**
     * 添加岗位接口
     * @param map
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody SysPosition map) {
        return positionService.edit(map);
    }

    /**
     * 删除岗位接口
     * @param map
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(@RequestBody SysPosition map) {
        return positionService.drop(map.getId());
    }

    /**
     * 修改岗位状态
     * @param map
     * @return
     */
    @PostMapping("/changeStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response changeStatus(@RequestBody SysPosition map) {
        return positionService.changeStatus(map.getId(), map.getStatus());
    }

    /**
     * 获取岗位状态
     * @return
     */
    @PostMapping("/getStatus")
    public Response getStatus() {
        return response.success("操作成功", SysPositionConstant.getPositionStatus());
    }
}
