package com.taauav.admin.entity;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.taauav.common.domain.Entity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotEmpty;

/**
 * <p>
 * 权限规则表
 * </p>
 *
 * @author dyg
 * @since 2019-11-04
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("sys_auth_rule_new")
public class SysAuthRule extends Entity {

    private static final long serialVersionUID = 1L;

    /**
     * 规则唯一英文标识,全小写
     */
    @NotEmpty(message = "菜单地址不能为空")
    @TableField("`name`")
    @Length(max = 30, message = "菜单地址长度不能超过30个字")
    private String name;

    /**
     * 规则中文名称
     */
    @NotEmpty(message = "菜单标题不能为空")
    @Length(max = 10, message = "菜单标题长度不能超过10个字")
    private String title;

    /**
     * 权限
     */
    @NotEmpty(message = "权限不能为空")
    @Length(max = 50, message = "权限长度不能超过50个字")
    private String permission;

    /**
     * 父级id
     */
    private Integer pid;

    /**
     * 菜单层级
     */
    @TableField("`level`")
    private Integer level;

    /**
     * 有无子级：1有 2无
     */
    private Integer hasChild;

    /**
     * 分类：1操作 2菜单
     */
    @TableField("`type`")
    private Integer type;

    /**
     * 图标
     */
    @Length(max = 50, message = "图标长度不能超过50个字")
    private String icon;

    /**
     * 规则附加条件
     */
    @TableField("`condition`")
    private String condition;

    /**
     * 备注
     */
    @Length(max = 200, message = "备注不能超过200字")
    private String remark;

    /**
     * 排序
     */
    private Integer sort;

    public Map<Integer, String> getStatusList() {
        Map<Integer, String> map = new HashMap<>(2);
        map.put(1, "显示");
        map.put(2, "隐藏");
        return map;
    }
}
