package com.taauav.admin.service;

import com.taauav.admin.entity.SysCity;
import com.taauav.admin.entity.TauvInspectFile;
import com.taauav.admin.vo.LsAdminListVo;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.LsAdmin;
import com.baomidou.mybatisplus.extension.service.IService;
import com.taauav.admin.query.LsAdminQuery;
import com.taauav.common.service.IBaseService;
import com.taauav.front.dto.LsInspectFileDTO;
import com.taauav.front.vo.LSAdminVo;
import com.taauav.front.vo.LSVillageVo;
import com.taauav.front.vo.SysCityVo;

import javax.validation.constraints.Max;
import java.math.BigInteger;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 外部人员表 服务类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
public interface ILsAdminService extends IBaseService<LsAdmin> {

    /**
     * 获取用户列表
     *
     * @param query 查询条件
     * @return
     */
    Response getList(LsAdminQuery query);

    /**
     * 添加用户
     *
     * @param entity 实体对象
     * @return
     */
    Response add(LsAdmin entity);

    /**
     * 获取数据详情
     *
     * @param id 用户ID
     * @return
     */
    Response info(Integer id);

    /**
     * 编辑用户
     *
     * @param entity 实体对象
     * @return
     */
    Response edit(LsAdmin entity);

    /**
     * 删除用户
     *
     * @param id 用户ID
     * @return
     */
    Response delete(Integer id);

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    Response setStatus(LsAdmin entity);

    /**
     * 重置密码
     *
     * @param entity 实体对象
     * @return
     */
    Response resetPwd(LsAdmin entity);

    /**
     * 基础数据
     *
     * @return
     */
    Map<String, Object> getBaseData();

    /**
     * 获取用户登录权限
     *
     * @param userId 用户ID
     * @return
     */
    Map<Integer, String> getAdminAuthData(Integer userId);

    /**
     * 获取河长列表
     *
     * @return
     */
    Response getDriverManagerList();

    /**
     * 获取用户信息
     * @param userId
     * @return
     */
    LSAdminVo getAdminInfo(Integer userId);

    /**
     * 获取用户有查看权限区域id
     * @param adminInfo
     * @param map
     * @return
     */
    List<SysCityVo> getAdminAreaList(LSAdminVo adminInfo, Map<String, String> map);

    /**
     * 根据镇/街道id获取权限内下属村数据
     * @param userId
     * @param cityId
     * @param map
     * @return
     */
    LSVillageVo getVillageList(Integer userId, BigInteger cityId, Map<String, String> map);

    /**
     * 获取区域内所有河道问题坐标点
     * @param inspectFileDTO
     * @return
     */
    Response getQuestionList(LsInspectFileDTO inspectFileDTO);
}
