package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.constant.CommonConstant;
import com.taauav.admin.dto.TauvDriverDTO;
import com.taauav.admin.entity.LsAuthGroup;
import com.taauav.admin.entity.SysCity;
import com.taauav.admin.entity.TauvInspectFile;
import com.taauav.admin.mapper.LsAuthGroupMapper;
import com.taauav.admin.mapper.TauvInspectFileMapper;
import com.taauav.admin.service.*;
import com.taauav.admin.vo.LsAdminListVo;
import com.taauav.admin.vo.LsDriverManagerListVo;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.SysAdminConstant;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.RedisUtils;
import com.taauav.common.util.StringUtils;
import com.taauav.admin.entity.LsAdmin;
import com.taauav.admin.mapper.LsAdminMapper;
import com.taauav.admin.query.LsAdminQuery;
import com.taauav.front.dto.LsInspectFileDTO;
import com.taauav.front.vo.LSAdminVo;
import com.taauav.front.vo.LSVillageVo;
import com.taauav.front.vo.LsInspectFileVo;
import com.taauav.front.vo.SysCityVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 外部人员表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class LsAdminServiceImpl extends BaseServiceImpl<LsAdminMapper, LsAdmin> implements ILsAdminService {

    @Autowired
    private LsAdminMapper lsAdminMapper;
    @Autowired
    private Response response;
    @Autowired
    private ISysCityService cityService;
    @Autowired
    private ILsAuthGroupService authGroupService;
    @Autowired
    private LsAuthGroupMapper authGroupMapper;
    @Value("${server.IMAGE_URL}")
    private String imageUrl;
    @Autowired
    private ILsCityService lsCityService;
    @Autowired
    private TauvInspectFileMapper inspectFileMapper;
    @Autowired
    private ITauvDriverService driverService;
    @Autowired
    private RedisUtils redisUtils;

    /**
     * 获取数据列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(LsAdminQuery query) {
        // 查询条件
        IPage<LsAdmin> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<LsAdmin> data = lsAdminMapper.getAdminList(page, query);
        List<LsAdmin> lsAdminList = data.getRecords();
        List<LsAdminListVo> lsAdminListVoList = new ArrayList<>();
        if (!lsAdminList.isEmpty()) {
            lsAdminList.forEach(item -> {
                LsAdminListVo lsAdminListVo = new LsAdminListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, lsAdminListVo);
                // 区划描述
                String cityName = cityService.getCityName(item.getDriverArea());
                lsAdminListVo.setDriverAreaName(cityName);
                // 角色描述
                String authGroupName = authGroupService.getAuthGroupName(item.getAuthGroup());
                lsAdminListVo.setAuthGroupName(authGroupName);
                lsAdminListVoList.add(lsAdminListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", lsAdminListVoList);
        return response.success(result);
    }

    /**
     * 添加用户
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(LsAdmin entity) {
        if (StringUtils.isEmpty(entity.getPassword())) {
            // 初始密码:123456
            entity.setPassword(FunctionUtils.password("123456"));
        } else {
            entity.setPassword(FunctionUtils.password(entity.getPassword()));
        }
        boolean result = this.addData(entity);
        if (!result) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取用户详情
     *
     * @param id 用户ID
     * @return
     */
    @Override
    public Response info(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("用户ID不存在");
        }
        LsAdmin entity = lsAdminMapper.selectById(id);
        if (entity == null) {
            return response.failure("用户信息不存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑用户
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        // 设置密码并加密
        if (!StringUtils.isEmpty(entity.getPassword())) {
            entity.setPassword(null);
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除用户
     *
     * @param id 用户ID
     * @return
     */
    @Override
    public Response delete(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("用户ID不存在");
        }
        LsAdmin entity = lsAdminMapper.selectById(id);
        if (entity == null) {
            return response.failure("用户信息不存在");
        }
        boolean result = this.deleteById(id, entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("用户状态不能为空");
        }
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

    /**
     * 重置密码
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response resetPwd(LsAdmin entity) {
        if (entity.getId() == null || entity.getId() <= 0) {
            return response.failure("用户ID不存在");
        }
        // 重置密码：123456
        entity.setPassword(FunctionUtils.password("123456"));
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("重置失败");
        }
        return response.success("重置成功");
    }

    /**
     * 获取基础数据
     *
     * @return
     */
    @Override
    public Map<String, Object> getBaseData() {
        Map<String, Object> map = new HashMap<>(2);
        map.put("level_list", CommonConstant.LSADMIN_LEVEL_LIST);
        map.put("area_list", cityService.getCityList(BigInteger.valueOf(Long.valueOf("320117"))));
        return map;
    }

    /**
     * 获取人员数据权限
     *
     * @param userId 用户ID
     * @return
     */
    @Override
    public Map<Integer, String> getAdminAuthData(Integer userId) {
        LsAdmin adminInfo = lsAdminMapper.selectById(userId);
        if (adminInfo == null) {
            return null;
        }
        String[] authGroup = adminInfo.getAuthGroup().split(",");
        ArrayList arrayList = new ArrayList();
        for (String s : authGroup) {
            arrayList.add(s);
        }
        // 查询条件
        QueryWrapper<LsAuthGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", arrayList);
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);

        // 查询数据
        Map<Integer, String> result = new HashMap<>();
        List<LsAuthGroup> authGroupList = authGroupMapper.selectList(queryWrapper);
        if (!authGroupList.isEmpty()) {
            for (LsAuthGroup lsAuthGroup : authGroupList) {
                if (lsAuthGroup.getAuthData() == null || lsAuthGroup.getAuthData() <= 0) {
                    continue;
                }
                result.put(Integer.valueOf(lsAuthGroup.getAuthData()), lsAuthGroup.getDriverArea());
            }
        }
        return result;
    }

    /**
     * 获取河长列表
     *
     * @return
     */
    @Override
    public Response getDriverManagerList() {
        // 查询条件
        QueryWrapper<LsAdmin> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");
        queryWrapper.select("id", "realname");

        // 查询数据
        List<LsAdmin> lsAdminList = lsAdminMapper.selectList(queryWrapper);
        List<LsDriverManagerListVo> driverManagerListVoList = new ArrayList<>();
        if (!lsAdminList.isEmpty()) {
            lsAdminList.forEach(item -> {
                LsDriverManagerListVo driverManagerListVo = new LsDriverManagerListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, driverManagerListVo);
                driverManagerListVoList.add(driverManagerListVo);

            });
        }
        return response.success(driverManagerListVoList);
    }

    @Override
    public LSAdminVo getAdminInfo(Integer userId) {
        if (userId == null || userId <= 0) {
            return null;
        }
        LsAdmin admin = getInfo(userId);
        if (admin == null) {
            return null;
        }
        LSAdminVo info = new LSAdminVo();
        BeanUtils.copyProperties(admin, info);
        //处理头像
        if (!"".equals(info.getAvatar())) {
            info.setAvatar(imageUrl + info.getAvatar());
        }
        //处理角色名称
        String authText = authGroupService.getAuthGroupName(info.getAuthGroup());
        info.setAuthGroupName(authText);
        //处理状态名称
        String statusText = FunctionUtils.getArrayText(String.valueOf(info.getStatus()), SysAdminConstant.getAdminStatus());
        info.setStatusText(statusText);
        //处理区划数据
        if (info.getDriverArea() != null && info.getDriverArea().compareTo(BigInteger.ZERO) > 0) {
            SysCity city = cityService.getInfoById(info.getDriverArea());
            if (city != null) {
                info.setDriverAreaName(city.getName());
                info.setDriverAreaLevel(city.getLevel());
            }
        }

        return info;
    }

    /**
     * 获取用户权限内区、镇数据
     *
     * @param adminInfo
     * @param map
     * @return
     */
    @Override
    public List<SysCityVo> getAdminAreaList(LSAdminVo adminInfo, Map<String, String> map) {
        if (adminInfo == null) {
            return null;
        }
        if (map == null) {
            map = new HashMap<>();
        }
        String[] authGroup = adminInfo.getAuthGroup().split(",");
        ArrayList arrayList = new ArrayList();
        if (authGroup.length > 0) {
            for (String authId : authGroup) {
                arrayList.add(authId);
            }
        }
        // 查询角色信息
        QueryWrapper<LsAuthGroup> wrapper = new QueryWrapper<>();
        wrapper.in("id", arrayList);
        wrapper.eq("status", 1);
        wrapper.eq("mark", 1);
        List<LsAuthGroup> authGroupList = authGroupService.list(wrapper);
        List<SysCity> areaList = new ArrayList<>();
        List<SysCityVo> cityVoList = new ArrayList<>();
        if (null != authGroupList && authGroupList.size() > 0) {
            for (LsAuthGroup auth : authGroupList) {
                //用户自己区划
                if (auth.getAuthData().equals(1)) {
                    SysCity areaInfo = cityService.getInfoById(adminInfo.getDriverArea());
                    if (!areaList.contains(areaInfo) && areaInfo.getLevel() < 5) {
                        areaList.add(areaInfo);
                        cityVoList.add(countQuestion(areaInfo, map));
                    } else if (areaInfo.getLevel() >= 4) {
                        cityVoList = getCityParentList(adminInfo.getDriverArea(), cityVoList);
                    }
                } else if (auth.getAuthData().equals(2)) {
                    //用户自己区划
                    SysCity areaInfo = cityService.getInfoById(adminInfo.getDriverArea());
                    if (!areaList.contains(areaInfo) && areaInfo.getLevel() < 5) {
                        areaList.add(areaInfo);
                        cityVoList.add(countQuestion(areaInfo, map));
                    } else if (areaInfo.getLevel() >= 4) {
                        cityVoList = getCityParentList(adminInfo.getDriverArea(), cityVoList);
                    }
                    //用户子级区划
                    List<SysCity> cityList = getChildCityList(adminInfo.getDriverArea(), 5);
                    if (cityList != null && cityList.size() > 0) {
                        for (SysCity city : cityList) {
                            if (!areaList.contains(city)) {
                                areaList.add(city);
                                cityVoList.add(countQuestion(city, map));
                            }
                        }
                    }
                } else if (auth.getAuthData().equals(3)) {
                    //用户指定区划
                    String[] driverList = auth.getDriverArea().split(",");
                    if (driverList != null && driverList.length > 0) {
                        for (String driver : driverList) {
                            BigInteger areaId = new BigInteger(driver);
                            SysCity areaInfo = cityService.getInfoById(areaId);
                            if (!areaList.contains(areaInfo) && areaInfo.getLevel() < 5) {
                                areaList.add(areaInfo);
                                SysCityVo cityVo = new SysCityVo();
                                BeanUtils.copyProperties(areaInfo, cityVo);
                                cityVo.setSurfaceWater(0);
                                cityVo.setShoreLine(0);
                                cityVo.setOutfall(0);
                                cityVo.setWaterQuality(0);
                                cityVo.setIsTrue(0);
                                cityVo.setDriverList(new ArrayList<>());
                                if (cityVoList.contains(cityVo)) {
                                    cityVoList.remove(cityVo);
                                }
                                cityVoList.add(countQuestion(areaInfo, map));
                            } else if (areaInfo.getLevel() >= 4) {
                                cityVoList = getCityParentList(areaId, cityVoList);
                            }
                        }
                    }
                }
            }
        } else {
            return null;
        }

        //判断是否取过溧水区 city_id = 320117
        Boolean exist = false;
        BigInteger lsId = new BigInteger("320117");
        if (cityVoList.size() > 0) {
            for (SysCityVo cityVo : cityVoList) {
                if (cityVo.getId().equals(lsId)) {
                    exist = true;
                    break;
                }
            }
        }
        if (!exist) {
            SysCity lishui = cityService.getInfoById(lsId);
            SysCityVo lsVo = new SysCityVo();
            BeanUtils.copyProperties(lishui, lsVo);
            lsVo.setSurfaceWater(0);
            lsVo.setShoreLine(0);
            lsVo.setOutfall(0);
            lsVo.setWaterQuality(0);
            lsVo.setIsTrue(0);
            lsVo.setDriverList(new ArrayList<>());
            cityVoList.add(lsVo);
        }

        return cityVoList;
    }

    /**
     * 获取限定层级内的子区划数组
     *
     * @param cityId
     * @param limitLevel
     * @return
     */
    private List<SysCity> getChildCityList(BigInteger cityId, Integer limitLevel) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        wrapper.eq("pid", cityId);
        if (limitLevel > 0) {
            wrapper.lt("level", limitLevel);
        }
        List<SysCity> cityList = cityService.list(wrapper);
        return cityList;
    }

    /**
     * 根据城市id获取村级以上城市数据
     *
     * @param cityId
     * @param cityList
     * @return
     */
    private List<SysCityVo> getCityParentList(BigInteger cityId, List<SysCityVo> cityList) {
        List<SysCity> parentList = cityService.getParentCityList(cityId);
        if (parentList != null && parentList.size() > 0) {
            for (SysCity item : parentList) {
                SysCityVo cityVo = new SysCityVo();
                BeanUtils.copyProperties(item, cityVo);
                cityVo.setSurfaceWater(0);
                cityVo.setShoreLine(0);
                cityVo.setOutfall(0);
                cityVo.setWaterQuality(0);
                cityVo.setIsTrue(0);
                cityVo.setDriverList(new ArrayList<>());
                Boolean exist = false;
                if (cityList != null && cityList.size() > 0) {
                    for (SysCityVo val : cityList) {
                        if (cityVo.getId().equals(val.getId())) {
                            exist = true;
                        }
                    }
                }
                if (!exist) {
                    cityList.add(cityVo);
                }
            }
        }
        return cityList;
    }

    /**
     * 计算区划问题数量
     *
     * @param city
     * @param map
     * @return
     */
    private SysCityVo countQuestion(SysCity city, Map<String, String> map) {
        //计算问题数量
        SysCityVo cityVo = countQuestionNum(city, map);
        cityVo.setIsTrue(1);
        //获取区划内河道数据
        List<TauvDriverDTO> driverDTOList = getDriverList(cityVo.getId());
        cityVo.setDriverList(driverDTOList);
        return cityVo;
    }

    /**
     * 根据镇/街道id获取权限内下属村数据
     *
     * @param userId
     * @param townId
     * @param map
     * @return
     */
    @Override
    public LSVillageVo getVillageList(Integer userId, BigInteger townId, Map<String, String> map) {
        LsAdmin adminInfo = getInfo(userId);
        if (adminInfo == null) {
            return null;
        }

        ArrayList arrayList = new ArrayList();
        String[] authGroup = adminInfo.getAuthGroup().split(",");
        if (authGroup != null && authGroup.length > 0) {
            for (String authId : authGroup) {
                arrayList.add(authId);
            }
        }
        // 查询角色信息
        QueryWrapper<LsAuthGroup> wrapper = new QueryWrapper<>();
        wrapper.in("id", arrayList);
        wrapper.eq("status", 1);
        wrapper.eq("mark", 1);
        List<LsAuthGroup> authGroupList = authGroupService.list(wrapper);
        String key = "userId_" + userId + "_townId_" + townId + "_village_driver";
        Boolean exist = redisUtils.hasKey(key);
        List<TauvDriverDTO> driverDTOList = new ArrayList<>();
        Boolean permission = false;
        if (exist) {
            Object object = redisUtils.get(key);
            driverDTOList = FunctionUtils.castList(object, TauvDriverDTO.class);
        }
        List<SysCityVo> cityVoList = new ArrayList<>();
        List<SysCity> cityList = new ArrayList<>();
        if (null != authGroupList && authGroupList.size() > 0) {
            for (LsAuthGroup auth : authGroupList) {
                SysCity city = new SysCity();
                switch (auth.getAuthData()) {
                    case 1:
                        city = cityService.getInfoById(adminInfo.getDriverArea());
                        if (city.getLevel() == 5 && !cityList.contains(city) && city.getPid().equals(townId)) {
                            cityList.add(city);
                            permission = true;
                            cityVoList.add(countQuestionNum(city, map));
                            if (!exist) {
                                driverDTOList = dealDriverList(city.getId(), driverDTOList);
                            }
                        }
                        break;
                    case 2:
                        city = cityService.getInfoById(adminInfo.getDriverArea());
                        if (city.getLevel() == 5 && !cityList.contains(city) && city.getPid().equals(townId)) {
                            cityList.add(city);
                            permission = true;
                            cityVoList.add(countQuestionNum(city, map));
                            if (!exist) {
                                driverDTOList = dealDriverList(city.getId(), driverDTOList);
                            }
                        } else if (city.getLevel() == 4 && townId.equals(adminInfo.getDriverArea())) {
                            permission = true;
                            //获取子级
                            List<SysCity> childCityList = getChildCityList(townId, 0);
                            for (SysCity item : childCityList) {
                                if (!cityList.contains(item)) {
                                    cityList.add(item);
                                    cityVoList.add(countQuestionNum(item, map));
                                    if (!exist) {
                                        driverDTOList = dealDriverList(city.getId(), driverDTOList);
                                    }
                                }
                            }
                        } else if (city.getLevel() == 3) {
                            List<SysCity> townList = getChildCityList(city.getId(), 0);
                            List<BigInteger> townIds = new ArrayList<>();
                            if (townList != null && townList.size() > 0) {
                                for (SysCity item : townList) {
                                    townIds.add(item.getId());
                                }
                            }
                            if (townIds.contains(townId)) {
                                permission = true;
                                List<SysCity> childCityList = getChildCityList(townId, 0);
                                for (SysCity item : childCityList) {
                                    if (!cityList.contains(item)) {
                                        cityList.add(item);
                                        cityVoList.add(countQuestionNum(item, map));
                                        if (!exist) {
                                            driverDTOList = dealDriverList(city.getId(), driverDTOList);
                                        }
                                    }
                                }
                            }
                        }
                        break;
                    case 3:
                        //用户指定区划
                        String[] driverAreaList = auth.getDriverArea().split(",");
                        if (driverAreaList != null && driverAreaList.length > 0) {
                            for (String area : driverAreaList) {
                                BigInteger areaId = new BigInteger(area);
                                city = cityService.getInfoById(areaId);
                                if (city.getLevel().equals(5) && !cityList.contains(city) && city.getPid().equals(townId)) {
                                    cityList.add(city);
                                    permission = true;
                                    cityVoList.add(countQuestionNum(city, map));
                                    if (!exist) {
                                        driverDTOList = dealDriverList(city.getId(), driverDTOList);
                                    }
                                }
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        if (!exist && driverDTOList.size() > 0) {
            redisUtils.set(key, driverDTOList, 86400);
        }
        LSVillageVo villageVo = new LSVillageVo();
        villageVo.setCityVoList(cityVoList);
        villageVo.setDriverDTOList(driverDTOList);
        villageVo.setPermission(permission);
        return villageVo;
    }

    /**
     * 获取区划内所有河道问题坐标点
     *
     * @param inspectFileDTO
     * @return
     */
    @Override
    public Response getQuestionList(LsInspectFileDTO inspectFileDTO) {
        if (inspectFileDTO == null || inspectFileDTO.getCityId() == null || inspectFileDTO.getCategory() == null) {
            return response.failure("参数不能为空");
        }
        if (inspectFileDTO.getCityId() == null || inspectFileDTO.getCityId().compareTo(BigInteger.ZERO) <= 0) {
            return response.failure("区划id不合法");
        }
        if (inspectFileDTO.getCategory() == null || inspectFileDTO.getCategory() < 0) {
            return response.failure("问题类型不正确");
        }
        List<LsInspectFileVo> fileVoList = inspectFileMapper.getInspectFileList(inspectFileDTO);
        return response.success(fileVoList);
    }

    /**
     * 计算问题数量
     *
     * @param city
     * @param map
     * @return
     */
    private SysCityVo countQuestionNum(SysCity city, Map<String, String> map) {
        List<Map<String, Integer>> questionList = inspectFileMapper.countQuestionByArea(city.getId(), map);
        SysCityVo cityVo = new SysCityVo();
        BeanUtils.copyProperties(city, cityVo);
        if (questionList.size() > 0) {
            for (Map<String, Integer> question : questionList) {
                Number number = (Number) question.get("number");
                switch (question.get("category")) {
                    case 1:
                        cityVo.setSurfaceWater(number.intValue());
                        break;
                    case 2:
                        cityVo.setShoreLine(number.intValue());
                        break;
                    case 3:
                        cityVo.setOutfall(number.intValue());
                        break;
                    case 4:
                        cityVo.setWaterQuality(number.intValue());
                        break;
                    default:
                        break;
                }
            }
        }
        if (cityVo.getSurfaceWater() == null) {
            cityVo.setSurfaceWater(0);
        }
        if (cityVo.getShoreLine() == null) {
            cityVo.setShoreLine(0);
        }
        if (cityVo.getOutfall() == null) {
            cityVo.setOutfall(0);
        }
        if (cityVo.getWaterQuality() == null) {
            cityVo.setWaterQuality(0);
        }
        return cityVo;
    }

    /**
     * 获取河道数据
     *
     * @param cityId
     * @return
     */
    private List<TauvDriverDTO> getDriverList(BigInteger cityId) {
        List<TauvDriverDTO> driverList = driverService.getDriverListByCityId(cityId);
        if (driverList == null) {
            driverList = new ArrayList<>();
        }
        return driverList;
    }

    /**
     * 处理河道数据
     *
     * @param cityId
     * @param driverDTOList
     * @return
     */
    private List<TauvDriverDTO> dealDriverList(BigInteger cityId, List<TauvDriverDTO> driverDTOList) {
        List<TauvDriverDTO> dtoList = getDriverList(cityId);
        if (dtoList != null && dtoList.size() > 0) {
            for (TauvDriverDTO driver : dtoList) {
                if (!driverDTOList.contains(driver)) {
                    driverDTOList.add(driver);
                }
            }
        }
        return driverDTOList;
    }
}
