package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.admin.entity.SysCity;
import com.taauav.admin.mapper.SysCityMapper;
import com.taauav.admin.service.ILsCityService;
import com.taauav.admin.vo.LsCityListVo;
import com.taauav.common.bean.Response;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.ShiroUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 用户组表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class LsCityServiceImpl extends ServiceImpl<SysCityMapper, SysCity> implements ILsCityService {

    @Autowired
    private SysCityMapper sysCityMapper;

    @Autowired
    private Response response;

    /**
     * 获取机构列表
     *
     * @return
     */
    @Override
    public Response getList() {
        // 查询条件
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", BigInteger.valueOf(320117));
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询条件
        SysCity cityInfo = sysCityMapper.selectById(BigInteger.valueOf(320117));
        LsCityListVo cityListVo = new LsCityListVo();
        // 拷贝属性
        BeanUtils.copyProperties(cityInfo, cityListVo);
        // 获取子级
        List<LsCityListVo> cityList = this.getChildCityList(BigInteger.valueOf(320117));
        // 添加子级
        cityListVo.setChildrenList(cityList);

        // 返回结果
        List<LsCityListVo> cityListVoList = new ArrayList<>();
        cityListVoList.add(cityListVo);
        return response.success(cityListVoList);
    }

    /**
     * 获取子级城市列表
     *
     * @param pid 上级ID
     * @return
     */
    @Override
    public List<LsCityListVo> getChildCityList(BigInteger pid) {
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");
        List<SysCity> cityList = sysCityMapper.selectList(queryWrapper);
        List<LsCityListVo> cityListVoList = new ArrayList<>();
        if (!cityList.isEmpty()) {
            cityList.forEach(item -> {
                LsCityListVo cityListVo = new LsCityListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, cityListVo);
                // 判断是否有子级
                List<LsCityListVo> childrenList = this.getChildCityList(item.getId());
                if (!childrenList.isEmpty()) {
                    cityListVo.setChildrenList(childrenList);
                    cityListVo.setHasChildren(true);
                } else {
                    cityListVo.setHasChildren(false);
                }
                cityListVoList.add(cityListVo);
            });
        }
        return cityListVoList;
    }

    /**
     * 获取子级城市ID
     *
     * @param pid 上级城市
     * @return
     */
    @Override
    public List<BigInteger> getChildCityIds(BigInteger pid) {
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        List<BigInteger> cityIdsList = new ArrayList<>();
        List<SysCity> cityList = sysCityMapper.selectList(queryWrapper);
        if (!cityList.isEmpty()) {
            cityList.forEach(item -> {
                // 获取子级
                if (item.getLevel() <= 4) {
                    List<BigInteger> itemList = this.getChildCityIds(item.getId());
                    if (!itemList.isEmpty()) {
                        itemList.forEach(subItem -> {
                            cityIdsList.add(subItem);
                        });
                    }
                }
                cityIdsList.add(item.getId());
            });
        }
        return cityIdsList;
    }

    /**
     * 添加机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(SysCity entity) {
        if (entity == null) {
            return response.failure("实体对象不能为空");
        }
        if (entity.getPid() == null) {
            entity.setLevel(1);
        } else {
            SysCity agency = sysCityMapper.selectById(entity.getPid());
            if (entity != null) {
                entity.setLevel(agency.getLevel() + 1);
            }
        }
        entity.setCreateTime(DateUtil.now());
        entity.setCreateUser(ShiroUtils.getAdminId());
        int result = sysCityMapper.insert(entity);
        if (result == 0) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取机构详情
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response info(BigInteger id) {
        if (id == null) {
            return response.failure("机构ID不存在");
        }
        SysCity entity = this.getById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(SysCity entity) {
        if (entity == null || entity.getId() == null) {
            return response.failure("实体对象不能为空");
        }
        // 同名不给添加
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", entity.getName());
        queryWrapper.eq("level", entity.getLevel());
        queryWrapper.eq("mark", 1);
        SysCity cityInfo = sysCityMapper.selectOne(queryWrapper);
        if (cityInfo != null) {
            return response.failure("同级已存在该组织名称");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(DateUtil.now());
        boolean result = this.updateById(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除机构
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response delete(BigInteger id) {
        if (id == null) {
            return response.failure("机构ID不存在");
        }
        if (id == BigInteger.valueOf(Long.valueOf(320117))) {
            return response.failure("根级无法删除");
        }
        SysCity entity = sysCityMapper.selectById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        // 校验是否存在子级
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", entity.getId());
        queryWrapper.eq("mark", 1);
        Integer count = sysCityMapper.selectCount(queryWrapper);
        if (count > 0) {
            return response.failure("存在子级，无法删除");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(DateUtil.now());
        entity.setMark(0);
        boolean result = this.updateById(entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(SysCity entity) {
        if (entity.getId() == null) {
            return response.failure("设备ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("状态不存在");
        }
        int result = sysCityMapper.updateById(entity);
        if (result == 0) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

}
