package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysDict;
import com.taauav.admin.mapper.SysDictMapper;
import com.taauav.admin.service.ISysAdminService;
import com.taauav.admin.service.ISysDictService;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.Validator;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 系统字典表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-14
 */
@Service
public class SysDictServiceImpl extends BaseServiceImpl<SysDictMapper, SysDict> implements ISysDictService {
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Autowired
    private SysDictMapper dictMapper;
    @Resource
    private Validator validator;
    @Autowired
    private Response response;
    @Autowired
    private ISysAdminService adminService;

    /**
     * @param map
     * @return
     */
    @Override
    public Response getPageList(Map<String, String> map) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        wrapper.eq("pid", 0);
        Integer pageNo = 1;
        if (map != null) {
            String keywords = "keywords";
            if (map.containsKey(keywords)) {
                if (FunctionUtils.isNumeric(map.get(keywords))) {
                    wrapper.eq("id", Integer.valueOf(map.get(keywords)));
                } else if (!FunctionUtils.isEmpty(keywords, map)) {
                    wrapper.like("name", map.get(keywords));
                    wrapper.or();
                    wrapper.like("nid", map.get(keywords));
                }
            }
            //添加人筛选
            String createUser = "createUser";
            if (!FunctionUtils.isEmpty(createUser, map)) {
                wrapper.eq("`create_user`", Integer.valueOf(map.get(createUser)));
            }
            //添加时间筛选
            String startTime = "startTime";
            String endTime = "endTime";
            if (map.containsKey(startTime) && map.containsKey(endTime)) {
                wrapper.between("`create_time`", map.get(startTime), map.get(endTime));
            }
            pageNo = map.containsKey("page") && Integer.valueOf(map.get("page")) > 0 ? Integer.valueOf(map.get("page")) : 1;
            //每页条数
            String pgSize = "pageSize";
            if (map.containsKey(pgSize) && Integer.valueOf(map.get(pgSize)) > 0) {
                pageSize = Integer.valueOf(map.get(pgSize));
            }
        }
        IPage<SysDict> page = new Page<>(pageNo, pageSize);
        IPage<Map<String, Object>> data = dictMapper.selectMapsPage(page, wrapper);
        if (!StringUtils.isEmpty(data.getRecords())) {
            for (Map<String, Object> mp : data.getRecords()) {
                Integer createUser = new Long(mp.get("create_user").toString()).intValue();
                mp.put("createUserName", adminService.getRealNameById(createUser));

                mp.put("formatCreateTime", mp.get("create_time").toString());
                if ((Integer) mp.get("has_child") == 1) {
                    QueryWrapper query = new QueryWrapper();
                    query.eq("mark", 1);
                    query.eq("pid", mp.get("id"));
                    List<Map<String, Object>> fChild = listMaps(query);
                    if (fChild != null) {
                        for (Map<String, Object> val : fChild) {
                            createUser = new Long(val.get("create_user").toString()).intValue();
                            val.put("createUserName", adminService.getRealNameById(createUser));
                            val.put("formatCreateTime", val.get("create_time").toString());
                            if ((Integer) val.get("has_child") == 1) {
                                QueryWrapper cond = new QueryWrapper();
                                cond.eq("pid", val.get("id"));
                                cond.eq("mark", 1);
                                List<Map<String, Object>> sChild = listMaps(cond);
                                if (sChild != null) {
                                    for (Map<String, Object> item : sChild) {
                                        createUser = new Long(item.get("create_user").toString()).intValue();
                                        item.put("createUserName", adminService.getRealNameById(createUser));
                                        item.put("formatCreateTime", item.get("create_time").toString());
                                    }
                                    val.put("children", sChild);
                                }
                            }
                        }
                        mp.put("children", fChild);
                    }
                }
            }
        }
        return response.success("操作成功", data);
    }

    /**
     * 根据id获取数据
     *
     * @param id
     * @return
     */
    @Override
    public SysDict getDictById(Integer id) {
        SysDict dict = dictMapper.selectById(id);
        return dict;
    }

    /**
     * 添加、编辑数据
     *
     * @param map
     * @return
     */
    @Override
    public Response editData(SysDict map) {
        //必填字段及格式校验
        Set<ConstraintViolation<SysDict>> violations = validator.validate(map);
        for (ConstraintViolation<SysDict> violation : violations) {
            if (!"".equals(violation.getMessage())) {
                return response.failure(violation.getMessage());
            }
        }
        if (map.getId() != null && map.getId() > 0) {
            map.setUpdateTime(DateUtil.now());
            map.setUpdateUser(ShiroUtils.getAdminId());
            updateById(map);
        } else {
            map.setCreateTime(DateUtil.now());
            map.setCreateUser(ShiroUtils.getAdminId());
            save(map);
        }
        return response.success("操作成功");
    }

    /**
     * 根据类型获取数据列表
     *
     * @param typeId
     * @return
     */
    @Override
    public List<SysDict> getListByTypeId(Integer typeId) {
        QueryWrapper<SysDict> wrapper = new QueryWrapper<>();
        wrapper.eq("type_id", typeId);
        wrapper.eq("mark", 1);
        List<SysDict> list = dictMapper.selectList(wrapper);
        return list;
    }
}
