package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvWaterStandard;
import com.taauav.admin.mapper.TauvWaterStandardMapper;
import com.taauav.admin.query.TauvWaterStandardQuery;
import com.taauav.admin.service.ITauvWaterStandardService;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * 地表水环境质量标准 服务类
 *
 * @author daixiantong
 * @date 2020-05-13
 */
@Service
public class TauvWaterStandardServiceImpl extends BaseServiceImpl<TauvWaterStandardMapper, TauvWaterStandard> implements ITauvWaterStandardService {

    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Resource
    private Response response;

    /**
     * 自定义分类列表
     * @return
     */
    @Override
    public Response getPageList(TauvWaterStandardQuery query) {
        IPage<TauvWaterStandard> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvWaterStandard> data = baseMapper.selectPage(page, null);
        return response.success(data);
    }

    /**
     * 添加、编辑
     * @param waterStandard
     * @return
     */
    @Override
    public Response editStandard(TauvWaterStandard waterStandard) {
        if (StringUtils.isEmpty(waterStandard.getId().toString()) || waterStandard.getId() <= 0) {
            waterStandard.setCreateUser(ShiroUtils.getAdminId());
            waterStandard.setCreateTime(DateUtil.now());
        } else {
            waterStandard.setUpdateUser(ShiroUtils.getAdminId());
            waterStandard.setUpdateTime(DateUtil.now());
        }
        Boolean res = saveOrUpdate(waterStandard);
        if (!res) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }

    /**
     * 删除
     * @param waterStandard
     * @return
     */
    @Override
    public Response dropStandard(TauvWaterStandard waterStandard) {
        if (StringUtils.isEmpty(waterStandard.getId().toString()) || waterStandard.getId() <= 0) {
            return response.failure("质量标准id不能为空");
        }
        UpdateWrapper wrapper = new UpdateWrapper();
        wrapper.eq("id", waterStandard.getId());
        wrapper.set("mark", 0);
        update(wrapper);
        return response.success("操作成功");
    }
}
